package com.ningdatech.pmapi.performance.manage;

import java.math.BigDecimal;
import java.time.LocalDateTime;
import java.util.*;
import java.util.stream.Collectors;

import cn.hutool.core.map.MapUtil;
import com.ningdatech.pmapi.performance.enumration.AppIndexTypeEnum;
import com.ningdatech.pmapi.performance.model.dto.AppAppraisalInfoDTO;
import com.ningdatech.pmapi.performance.model.vo.*;
import com.ningdatech.pmapi.projectlib.enumeration.ProjectStatusEnum;
import com.ningdatech.pmapi.projectlib.model.entity.ProjectApplication;
import com.ningdatech.pmapi.projectlib.service.IProjectApplicationService;
import org.apache.commons.lang3.StringUtils;
import org.checkerframework.checker.nullness.qual.Nullable;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.google.common.collect.Lists;
import com.ningdatech.basic.exception.BizException;
import com.ningdatech.basic.function.VUtils;
import com.ningdatech.basic.model.PageVo;
import com.ningdatech.basic.util.StrPool;
import com.ningdatech.pmapi.performance.constant.BizConst;
import com.ningdatech.pmapi.performance.enumration.AppraisalTypeEnum;
import com.ningdatech.pmapi.performance.model.dto.ProjectAppraisalDTO;
import com.ningdatech.pmapi.performance.model.dto.ProjectAppraisalInfoDTO;
import com.ningdatech.pmapi.performance.model.entity.*;
import com.ningdatech.pmapi.performance.model.req.PerformanceAppraisalListReq;
import com.ningdatech.pmapi.performance.service.*;
import com.ningdatech.pmapi.projectlib.enumeration.ProjectTypeEnum;
import com.ningdatech.pmapi.projectlib.model.entity.Project;
import com.ningdatech.pmapi.projectlib.service.IProjectService;
import com.ningdatech.pmapi.sys.model.entity.Role;
import com.ningdatech.pmapi.sys.service.IRoleService;
import com.ningdatech.pmapi.user.security.auth.model.UserInfoDetails;
import com.ningdatech.pmapi.user.util.LoginUserUtil;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollUtil;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;

/**
 * AppAppraisalManage
 * @return
 * @author CMM
 * @since 2023/08/10 8:55
 */
@Component
@Slf4j
@AllArgsConstructor
public class AppAppraisalManage {

    private final IPerformanceAppraisalService performanceAppraisalService;
    private final IPerformanceAppraisalProjectService performanceAppraisalProjectService;
    private final IProjectService projectService;
    private final IRoleService roleService;
    private final IPerformanceAppraisalAppScoreInfoService performanceAppraisalAppScoreInfoService;
    private final IPerformanceAppraisalProjectGroupService groupService;
    private final IProjectApplicationService projectApplicationService;
    private final IPerformanceAppraisalApplicationService performanceAppraisalApplicationService;
    private final IPerformanceAppraisalAppIndicatorService performanceAppraisalAppIndicatorService;
    private final IProjectCoreBusinessIndicatorsService projectCoreBusinessIndicatorsService;
    /**
     * 复评-评价计划列表
     * @param req
     * @return
     */
    public PageVo<PerformanceAppraisalVO> list(PerformanceAppraisalListReq req) {
        UserInfoDetails user = LoginUserUtil.loginUserDetail();
        String employeeCode = user.getEmployeeCode();
        String regionCode = user.getRegionCode();
        // 获取当前登录用户的角色列表，只有绩效复评人员才能查看该列表
        List<Role> userRoleList = user.getUserRoleList();
        List<Long> roleIdList = userRoleList.stream().map(Role::getId).collect(Collectors.toList());
        Role role = roleService.getOne(Wrappers.lambdaQuery(Role.class)
                .eq(Role::getName, BizConst.REAPPRAISAL_ROLE));
        if (Objects.isNull(role)){
            throw new BizException("登录用户没有复评员角色!");
        }

        // 登录用户不是复评员，不能查看复评计划列表
        if (!roleIdList.contains(role.getId())){
            return PageVo.empty();
        }

        // 获取登录用户所在区域的所有已终验项目信息
        List<Project> projectLists = projectService.list(Wrappers.lambdaQuery(Project.class)
                .eq(Project::getStatus, ProjectStatusEnum.ACCEPTED.getCode())
                .eq(Project::getAreaCode, regionCode));
        List<String> projectCodes = projectLists.stream().map(Project::getProjectCode).collect(Collectors.toList());
        // 获取项目关联的应用信息
        List<ProjectApplication> applications = projectApplicationService.list(Wrappers.lambdaQuery(ProjectApplication.class)
                .in(ProjectApplication::getProjectCode, projectCodes));
        // 已验收的项目关联的应用信息不存在
        if(CollUtil.isEmpty(applications)){
            return PageVo.empty();
        }
        List<Long> appIds = applications.stream().map(ProjectApplication::getId).collect(Collectors.toList());
        List<PerformanceAppraisalApplication> paas = performanceAppraisalApplicationService.list(Wrappers.lambdaQuery(PerformanceAppraisalApplication.class)
                .in(PerformanceAppraisalApplication::getApplicationId, appIds));
        // 没有包含应用的评价计划
        if (CollUtil.isEmpty(paas)){
            return PageVo.empty();
        }
        Set<Long> paIds = paas.stream().map(PerformanceAppraisalApplication::getAppraisalId).collect(Collectors.toSet());

        // 获取评价计划关联的分组信息
        List<PerformanceAppraisalProjectGroup> groups = groupService.list(Wrappers.lambdaQuery(PerformanceAppraisalProjectGroup.class)
                .in(PerformanceAppraisalProjectGroup::getAppraisalId, paIds));
        // 获取当前登录复评员所在的分组信息
        List<PerformanceAppraisalProjectGroup> groupList = groups.stream().filter(g -> {
            String reAppraisalUsers = g.getReAppraisalUsers();
            List<String> empCodeList = Lists.newArrayList();
            if (reAppraisalUsers.contains(StrPool.COMMA)) {
                empCodeList.addAll(Arrays.asList(reAppraisalUsers.split(StrPool.COMMA)));
            }else {
                empCodeList.add(reAppraisalUsers);
            }
            if (empCodeList.contains(employeeCode)) {
                return true;
            }
            return false;
        }).collect(Collectors.toList());
        // 当前登录复评员用户，没有参加评价计划
        if (CollUtil.isEmpty(groupList)){
            return PageVo.empty();
        }
        // 获取分组所在的评价计划ID
        List<Long> appraisalIds = groupList.stream().map(PerformanceAppraisalProjectGroup::getAppraisalId).collect(Collectors.toList());
        // 获取复评计划列表
        Page<PerformanceAppraisal> page = req.page();
        LambdaQueryWrapper<PerformanceAppraisal> wrapper = Wrappers.lambdaQuery(PerformanceAppraisal.class)
                .in(PerformanceAppraisal::getId,appraisalIds)
                .orderByDesc(PerformanceAppraisal::getUpdateOn);
        performanceAppraisalService.page(page,wrapper);
        List<PerformanceAppraisalVO> res = page.getRecords().stream()
                .map(p -> BeanUtil.copyProperties(p, PerformanceAppraisalVO.class))
                .collect(Collectors.toList());
        return PageVo.of(res,page.getTotal());
    }


    public PageVo<PerformanceAppraisalAppVO> unEvaluateList(Long planId, PerformanceAppraisalListReq req) {
        UserInfoDetails user = LoginUserUtil.loginUserDetail();
        String employeeCode = user.getEmployeeCode();

        PerformanceAppraisal plan = performanceAppraisalService.getById(planId);
        VUtils.isTrue(Objects.isNull(plan)).throwMessage("该评价计划不存在!");

        // 获取评价计划内已添加的未完成评价应用信息，且应用为当前登录复评用户所在分组评价项目关联的应用
        List<PerformanceAppraisalProjectGroup> groupList = groupService.list(Wrappers.lambdaQuery(PerformanceAppraisalProjectGroup.class)
                .eq(PerformanceAppraisalProjectGroup::getAppraisalId, planId));
        // 筛选出当前登录复评用户所在的分组信息
        List<PerformanceAppraisalProjectGroup> groups = groupList.stream().filter(g -> {
            String reAppraisalUsers = g.getReAppraisalUsers();
            if (reAppraisalUsers.contains(employeeCode)) {
                return true;
            }
            return false;
        }).collect(Collectors.toList());
        // 获取评价计划分组信息内的项目信息
        List<String> proCodes = groups.stream()
                .map(PerformanceAppraisalProjectGroup::getProjectCodes)
                .collect(Collectors.toList());
        List<String> projectCodes = Lists.newArrayList();
        for (String proCode : proCodes) {
            if (proCode.contains(StrPool.COMMA)){
                projectCodes.addAll(Arrays.asList(proCode.split(StrPool.COMMA)));
            }else {
                projectCodes.add(proCode);
            }
        }

        List<PerformanceAppraisalApplication> paas = performanceAppraisalApplicationService.list(Wrappers.lambdaQuery(PerformanceAppraisalApplication.class)
                .eq(PerformanceAppraisalApplication::getAppraisalId, plan.getId())
                .in(PerformanceAppraisalApplication::getAppraisalProjectCode,projectCodes)
                .eq(PerformanceAppraisalApplication::getIsCompleteAppraisal,Boolean.FALSE));
        if (CollUtil.isEmpty(paas)){
            return PageVo.empty();
        }
        Set<Long> appraisalIds = paas.stream()
                .map(PerformanceAppraisalApplication::getAppraisalId)
                .collect(Collectors.toSet());
        // 从绩效评价打分信息表中获取上述绩效评价计划应用的打分信息
        List<PerformanceAppraisalAppScoreInfo> appraisalScoreInfos = performanceAppraisalAppScoreInfoService.list(Wrappers.lambdaQuery(PerformanceAppraisalAppScoreInfo.class)
                .in(PerformanceAppraisalAppScoreInfo::getAppraisalId, appraisalIds));
        // 筛选出当前登录用户未进行打分的应用信息
        List<Long> appIds = appraisalScoreInfos.stream()
                .filter(a -> !employeeCode.equals(a.getAppraisalEmployeeCode()))
                .map(PerformanceAppraisalAppScoreInfo::getApplicationId)
                .collect(Collectors.toList());

        // 没有当前用户待评价的应用信息
        if(CollUtil.isEmpty(appIds)){
            return PageVo.empty();
        }

        Page<ProjectApplication> page = req.page();
        LambdaQueryWrapper<ProjectApplication> wrapper = Wrappers.lambdaQuery(ProjectApplication.class)
                .in(ProjectApplication::getId, appIds)
                .like(StringUtils.isNotBlank(req.getApplicationName()),ProjectApplication::getApplicationName,req.getApplicationName());
        projectApplicationService.page(page,wrapper);

        if(0L == page.getTotal()){
            return PageVo.empty();
        }

        List<PerformanceAppraisalAppVO> res = page.getRecords().stream()
                .map(p -> {
                    PerformanceAppraisalAppVO vo = BeanUtil.copyProperties(p, PerformanceAppraisalAppVO.class);
                    vo.setCanAppraisal(checkCanAppraisal(employeeCode,p.getId(),planId));
                    vo.setAppraisalId(planId);
                    return vo;
                })
                .collect(Collectors.toList());

        return PageVo.of(res,page.getTotal());
    }

    public PageVo<PerformanceAppraisalAppVO> evaluatedList(Long planId, PerformanceAppraisalListReq req) {

        UserInfoDetails user = LoginUserUtil.loginUserDetail();
        String employeeCode = user.getEmployeeCode();

        PerformanceAppraisal plan = performanceAppraisalService.getById(planId);
        VUtils.isTrue(Objects.isNull(plan)).throwMessage("该评价计划不存在!");


        // 获取评价计划内已添加的未完成评价应用信息，且应用为当前登录复评用户所在分组评价项目关联的应用
        List<PerformanceAppraisalProjectGroup> groupList = groupService.list(Wrappers.lambdaQuery(PerformanceAppraisalProjectGroup.class)
                .eq(PerformanceAppraisalProjectGroup::getAppraisalId, planId));
        // 筛选出当前登录复评用户所在的分组信息
        List<PerformanceAppraisalProjectGroup> groups = groupList.stream().filter(g -> {
            String reAppraisalUsers = g.getReAppraisalUsers();
            if (reAppraisalUsers.contains(employeeCode)) {
                return true;
            }
            return false;
        }).collect(Collectors.toList());
        // 获取评价计划分组信息内的项目信息
        List<String> proCodes = groups.stream()
                .map(PerformanceAppraisalProjectGroup::getProjectCodes)
                .collect(Collectors.toList());
        List<String> projectCodes = Lists.newArrayList();
        for (String proCode : proCodes) {
            if (proCode.contains(StrPool.COMMA)){
                projectCodes.addAll(Arrays.asList(proCode.split(StrPool.COMMA)));
            }else {
                projectCodes.add(proCode);
            }
        }

        List<PerformanceAppraisalApplication> paas = performanceAppraisalApplicationService.list(Wrappers.lambdaQuery(PerformanceAppraisalApplication.class)
                .eq(PerformanceAppraisalApplication::getAppraisalId, plan.getId())
                .in(PerformanceAppraisalApplication::getAppraisalProjectCode,projectCodes)
                .eq(PerformanceAppraisalApplication::getIsCompleteAppraisal,Boolean.FALSE));
        if (CollUtil.isEmpty(paas)){
            return PageVo.empty();
        }
        Set<Long> appraisalIds = paas.stream()
                .map(PerformanceAppraisalApplication::getAppraisalId)
                .collect(Collectors.toSet());
        // 从绩效评价打分信息表中获取上述绩效评价计划应用的打分信息
        List<PerformanceAppraisalAppScoreInfo> appraisalScoreInfos = performanceAppraisalAppScoreInfoService.list(Wrappers.lambdaQuery(PerformanceAppraisalAppScoreInfo.class)
                .in(PerformanceAppraisalAppScoreInfo::getAppraisalId, appraisalIds));
        // 筛选出当前登录用户未进行打分的应用信息
        List<Long> appIds = appraisalScoreInfos.stream()
                .filter(a -> employeeCode.equals(a.getAppraisalEmployeeCode()))
                .map(PerformanceAppraisalAppScoreInfo::getApplicationId)
                .collect(Collectors.toList());

        // 没有当前用户待评价的应用信息
        if(CollUtil.isEmpty(appIds)){
            return PageVo.empty();
        }

        Page<ProjectApplication> page = req.page();
        LambdaQueryWrapper<ProjectApplication> wrapper = Wrappers.lambdaQuery(ProjectApplication.class)
                .in(ProjectApplication::getId, appIds)
                .like(StringUtils.isNotBlank(req.getApplicationName()),ProjectApplication::getApplicationName,req.getApplicationName());
        projectApplicationService.page(page,wrapper);

        if(0L == page.getTotal()){
            return PageVo.empty();
        }

        List<PerformanceAppraisalAppVO> res = page.getRecords().stream()
                .map(p -> {
                    PerformanceAppraisalAppVO vo = BeanUtil.copyProperties(p, PerformanceAppraisalAppVO.class);
                    vo.setCanAppraisal(checkCanAppraisal(employeeCode,p.getId(),planId));
                    vo.setAppraisalId(planId);
                    return vo;
                })
                .collect(Collectors.toList());

        return PageVo.of(res,page.getTotal());
    }

    /**
     * 是否可以评价
     *
     * @param employeeCode
     * @param appId
     * @param appraisalId
     * @return
     */
    private Boolean checkCanAppraisal(String employeeCode,  Long appId, Long appraisalId) {
        // 从绩效评价应用打分信息表中查询打分人员为当前登录复评用户的打分信息
        // 如果没有相关的打分信息，说明是当前登录复评用户是首次评价
        List<PerformanceAppraisalAppScoreInfo> scoreInfos = performanceAppraisalAppScoreInfoService.list(Wrappers.lambdaQuery(PerformanceAppraisalAppScoreInfo.class)
                .eq(PerformanceAppraisalAppScoreInfo::getApplicationId,appId)
                .eq(PerformanceAppraisalAppScoreInfo::getAppraisalId,appraisalId)
                .eq(PerformanceAppraisalAppScoreInfo::getAppraisalEmployeeCode, employeeCode));
        if (CollUtil.isEmpty(scoreInfos)){
            return true;
        }
        return Boolean.FALSE;
    }


    public AppAppraisalIndexDetailVO detail(Long appId) {
        UserInfoDetails user = LoginUserUtil.loginUserDetail();
        String regionCode = user.getRegionCode();

        AppAppraisalIndexDetailVO resVo = new AppAppraisalIndexDetailVO();

        // 获取指标配置的应用指标信息
        // 登录复评用户所在区域通用指标
        // todo 是否需要过滤展示形式为打分的通用指标
        List<PerformanceAppraisalAppIndicator> commonIndexList = performanceAppraisalAppIndicatorService.list(Wrappers.lambdaQuery(PerformanceAppraisalAppIndicator.class)
                .eq(PerformanceAppraisalAppIndicator::getRegionCode, regionCode)
                .eq(PerformanceAppraisalAppIndicator::getIndexType, AppIndexTypeEnum.COMMON.getCode()));
        if (CollUtil.isNotEmpty(commonIndexList)) {
            List<AppIndexDetailVO> commonIndexDetails = commonIndexList.stream().map(i -> {
                AppIndexDetailVO vo = new AppIndexDetailVO();
                BeanUtils.copyProperties(i, vo);
                // 如果进行了打分，按复评人员分组
                List<PerformanceAppraisalAppScoreInfo> appAppraisalScoreInfo = performanceAppraisalAppScoreInfoService.list(Wrappers.lambdaQuery(PerformanceAppraisalAppScoreInfo.class)
                        .eq(PerformanceAppraisalAppScoreInfo::getApplicationId, appId));
                if (CollUtil.isNotEmpty(appAppraisalScoreInfo)){
                    Map<String, List<PerformanceAppraisalAppScoreInfo>> scoreInfoMap = appAppraisalScoreInfo.stream()
                            .collect(Collectors.groupingBy(PerformanceAppraisalAppScoreInfo::getAppraisalEmployeeName));
                    HashMap<String, BigDecimal> scoreMap = MapUtil.newHashMap();
                    for (String name : scoreInfoMap.keySet()) {
                        List<PerformanceAppraisalAppScoreInfo> scoreInfos = scoreInfoMap.get(name);
                        BigDecimal totalScore = scoreInfos.stream().map(PerformanceAppraisalAppScoreInfo::getAppraisalScore)
                                .filter(Objects::nonNull)
                                .reduce(BigDecimal.ZERO, BigDecimal::add);
                        scoreMap.put(name, totalScore);
                    }
                    resVo.setAppraisalScoreInfo(scoreInfoMap);
                    resVo.setAppraisalTotalScoreInfo(scoreMap);
                }
                return vo;
            }).collect(Collectors.toList());
            resVo.setCommonIndexDetails(commonIndexDetails);
        }

        // 登录复评用户所在区域辅助指标
        // todo 是否需要过滤展示形式为打分的辅助指标
        List<PerformanceAppraisalAppIndicator> auxIndexList = performanceAppraisalAppIndicatorService.list(Wrappers.lambdaQuery(PerformanceAppraisalAppIndicator.class)
                .eq(PerformanceAppraisalAppIndicator::getRegionCode, regionCode)
                .eq(PerformanceAppraisalAppIndicator::getIndexType, AppIndexTypeEnum.AUX.getCode()));
        if (CollUtil.isNotEmpty(auxIndexList)) {
            List<AppIndexDetailVO> auxIndexDetails = auxIndexList.stream().map(i -> {
                AppIndexDetailVO vo = new AppIndexDetailVO();
                BeanUtils.copyProperties(i, vo);
                return vo;
            }).collect(Collectors.toList());
            resVo.setAuxIndexDetails(auxIndexDetails);
        }
        // 获取应用的核心业务指标
        List<ProjectCoreBusinessIndicators> coreBusinessIndicators = projectCoreBusinessIndicatorsService.list(Wrappers.lambdaQuery(ProjectCoreBusinessIndicators.class)
                .eq(ProjectCoreBusinessIndicators::getApplicationId, appId));
        if (CollUtil.isNotEmpty(coreBusinessIndicators)) {
            List<AppBusinessIndexDetailVO> businessIndexDetails = coreBusinessIndicators.stream().map(c -> {
                AppBusinessIndexDetailVO appBusinessIndexDetailVo = new AppBusinessIndexDetailVO();
                BeanUtils.copyProperties(coreBusinessIndicators, appBusinessIndexDetailVo);
                return appBusinessIndexDetailVo;
            }).collect(Collectors.toList());
            resVo.setBusinessIndexDetails(businessIndexDetails);
        }
        return resVo;
    }

    public String saveAppraisal(ProjectAppraisalDTO param) {
        UserInfoDetails userDetail = LoginUserUtil.loginUserDetail();
        String employeeCode = userDetail.getEmployeeCode();
        String username = userDetail.getUsername();

        Long applicationId = param.getApplicationId();
        Long appraisalId = param.getAppraisalId();
        List<AppAppraisalInfoDTO> appAppraisalInfoList = param.getAppAppraisalInfoList();

        // 获取评价应用
        PerformanceAppraisalApplication appraisalApplication = performanceAppraisalApplicationService.getOne(Wrappers.lambdaQuery(PerformanceAppraisalApplication.class)
                .eq(PerformanceAppraisalApplication::getAppraisalId, appraisalId)
                .eq(PerformanceAppraisalApplication::getApplicationId, applicationId));

        List<Long> indexIds = appAppraisalInfoList.stream().filter(a -> AppIndexTypeEnum.COMMON.getCode().equals(a.getIndexType()) ||
                AppIndexTypeEnum.AUX.getCode().equals(a.getIndexType()))
                .map(AppAppraisalInfoDTO::getIndexId)
                .collect(Collectors.toList());
        List<Long> businessIndexIds = appAppraisalInfoList.stream()
                .filter(a -> AppIndexTypeEnum.BUSINESS.getCode().equals(a.getIndexType()))
                .map(AppAppraisalInfoDTO::getIndexId)
                .collect(Collectors.toList());
        // 查出通用、辅助应用指标分数信息
        List<PerformanceAppraisalAppIndicator> indexList = performanceAppraisalAppIndicatorService.list(Wrappers.lambdaQuery(PerformanceAppraisalAppIndicator.class)
                .in(PerformanceAppraisalAppIndicator::getId, indexIds));
        if (CollUtil.isEmpty(indexList)){
            throw new BizException("评价指标不存在!");
        }
        Map<Long, PerformanceAppraisalAppIndicator> indexMap = indexList.stream().collect(Collectors.toMap(PerformanceAppraisalAppIndicator::getId, d -> d));

        // 查出应用业务指标分数信息
        List<ProjectCoreBusinessIndicators> businessIndexList = projectCoreBusinessIndicatorsService.list(Wrappers.lambdaQuery(ProjectCoreBusinessIndicators.class)
                .in(ProjectCoreBusinessIndicators::getId, businessIndexIds));
        if (CollUtil.isEmpty(indexList)){
            throw new BizException("评价指标不存在!");
        }
        Map<Long, ProjectCoreBusinessIndicators> businessIndexMap = businessIndexList.stream().collect(Collectors.toMap(ProjectCoreBusinessIndicators::getId, d -> d));

        List<PerformanceAppraisalAppScoreInfo> scoreInfos = appAppraisalInfoList.stream().map(a -> {
            if (AppIndexTypeEnum.COMMON.getCode().equals(a.getIndexType()) ||
                    AppIndexTypeEnum.AUX.getCode().equals(a.getIndexType())){
                PerformanceAppraisalAppIndicator appIndicator = indexMap.get(a.getIndexId());
                BigDecimal score = a.getScore();
                BigDecimal indexScore = appIndicator.getIndexScore();
                if (score.compareTo(indexScore) > 0 || score.compareTo(BigDecimal.ZERO) < 0){
                    throw new BizException("评价得分必须大于等于0且小于等于指标分值!");
                }
            } else if (AppIndexTypeEnum.BUSINESS.getCode().equals(a.getIndexType())) {
                ProjectCoreBusinessIndicators coreBusinessIndicators = businessIndexMap.get(a.getIndexId());
                BigDecimal score = a.getScore();
                BigDecimal indexScore = coreBusinessIndicators.getScore();
                if (score.compareTo(indexScore) > 0 || score.compareTo(BigDecimal.ZERO) < 0){
                    throw new BizException("评价得分必须大于等于0且小于等于指标分值!");
                }
            }
            // 保存评价分数信息
            PerformanceAppraisalAppScoreInfo scoreInfo = new PerformanceAppraisalAppScoreInfo();
            scoreInfo.setAppraisalScore(a.getScore());
            scoreInfo.setAppraisalId(appraisalId);
            scoreInfo.setAppraisalEmployeeCode(employeeCode);
            scoreInfo.setAppraisalEmployeeName(username);
            scoreInfo.setApplicationId(applicationId);
            scoreInfo.setIndexId(a.getIndexId());
            scoreInfo.setIndexType(a.getIndexType());
            scoreInfo.setCreateOn(LocalDateTime.now());
            scoreInfo.setCreateBy(username);
            scoreInfo.setUpdateOn(LocalDateTime.now());
            scoreInfo.setUpdateBy(username);
            return scoreInfo;
        }).collect(Collectors.toList());
        if (performanceAppraisalAppScoreInfoService.saveBatch(scoreInfos)){
            // 计算评价总分并保存
            BigDecimal appraisalTotalScore = appAppraisalInfoList.stream()
                    .map(AppAppraisalInfoDTO::getScore)
                    .filter(Objects::nonNull)
                    .reduce(BigDecimal.ZERO, BigDecimal::add);
            BigDecimal lastAppraisalTotalScore;
            // 判断项目所在分组内其他复评人员是否已经打分
            if (Objects.nonNull(appraisalApplication.getAppraisalTotalScore())){
                // 已经进行了复评，计算两个分值的平均分
                BigDecimal add = appraisalTotalScore.add(appraisalApplication.getAppraisalTotalScore());
                lastAppraisalTotalScore = add.divide(BigDecimal.valueOf(2));
            }else {
                // 还没有进行过复评
                lastAppraisalTotalScore = appraisalTotalScore;
            }
            appraisalApplication.setAppraisalTotalScore(lastAppraisalTotalScore);
            // 取最新地打分时间
            appraisalApplication.setAppraisalScoreTime(LocalDateTime.now());
            // 判断当前复评人是否为该评价应用关联的评价项目所在分组的复评人员中的最后一个复评人
            // 获取评价应用关联的评价项目所在的评价计划关联的分组信息
            String appraisalProjectCode = appraisalApplication.getAppraisalProjectCode();
            List<PerformanceAppraisalProjectGroup> groups = groupService.list(Wrappers.lambdaQuery(PerformanceAppraisalProjectGroup.class)
                    .eq(PerformanceAppraisalProjectGroup::getAppraisalId,appraisalId));

            // 获取当前登录复评员所在包含该评价应用的分组信息
            List<PerformanceAppraisalProjectGroup> groupList = groups.stream().filter(g -> {
                String reAppraisalUsers = g.getReAppraisalUsers();
                String projectCodes = g.getProjectCodes();
                if (reAppraisalUsers.contains(employeeCode) &&
                        projectCodes.contains(appraisalProjectCode)) {
                    return true;
                }
                return false;
            }).collect(Collectors.toList());
            PerformanceAppraisalProjectGroup projectGroup = groupList.get(0);
            judgeCompleteAppraisal(projectGroup, appraisalApplication);
            appraisalApplication.setUpdateOn(LocalDateTime.now());
            appraisalApplication.setUpdateBy(username);
            performanceAppraisalApplicationService.updateById(appraisalApplication);
            return "提交成功";
        }
        return "提交失败";
    }


    private void judgeCompleteAppraisal(PerformanceAppraisalProjectGroup group, PerformanceAppraisalApplication appraisalApplication) {
        // 获取该评价应用的复评打分信息
        List<PerformanceAppraisalAppScoreInfo> scoreInfos = performanceAppraisalAppScoreInfoService.list(Wrappers.lambdaQuery(PerformanceAppraisalAppScoreInfo.class)
                .eq(PerformanceAppraisalAppScoreInfo::getAppraisalId, group.getAppraisalId())
                .eq(PerformanceAppraisalAppScoreInfo::getApplicationId, appraisalApplication.getApplicationId()));
        String reAppraisalUsers = group.getReAppraisalUsers();
        List<String> users = scoreInfos.stream().map(PerformanceAppraisalAppScoreInfo::getAppraisalEmployeeCode).collect(Collectors.toList());
        if (CollUtil.isNotEmpty(users)){
            String join = String.join(StrPool.COMMA, users);
            if (join.equals(reAppraisalUsers)) {
                appraisalApplication.setIsCompleteAppraisal(Boolean.TRUE);
            }
        }
    }
}
